﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading.Tasks;
using Abt.Controls.SciChart;
using Abt.Controls.SciChart.Model.DataSeries;

namespace SciChartTest
{
    public class ViewModel: INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void OnPropertyChanged([CallerMemberName] string propertyName = null)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (handler != null) handler(this, new PropertyChangedEventArgs(propertyName));
        }


        private bool _mouseWheelEnabled;
        private bool _panEnabled;
        private bool _rolloverEnabled;
        private bool _cursorEnabled;
        private bool _zoomEnabled;
        private IRange _sharedXVisibleRange;
        private readonly IDataSeries _chartData0;
        private readonly IDataSeries _chartData1;

        public ViewModel()
        {
            // Create two DataSeries' which we bind to in the view
            _chartData0 = CreateDataSeries1();
            _chartData1 = CreateDataSeries2();

            // Set default ChartModifier state
            this.MouseWheelEnabled = false;
            this.PanEnabled = false;
            this.CursorEnabled = true;
            ZoomEnabled = true;
            RolloverEnabled = false;

            // Set shared initial XAxis.VisibleRange. 
            // Note in code-behind we call sciChartSurface.AnimateZoomExtents(), which requires a
            // valid initial range, hence this binding
            SharedXVisibleRange = new DateRange(new DateTime(1990,1,1), new DateTime(1990,1,30));
            //SharedXVisibleRange = new DateRange(DateTime.MinValue, DateTime.MaxValue).GrowBy(1,1);
        }

        public IDataSeries ChartData0 { get { return _chartData0; } }
        public IDataSeries ChartData1 { get { return _chartData1; } }

        public IRange SharedXVisibleRange
        {
            get { return _sharedXVisibleRange; }
            set
            {
                if (_sharedXVisibleRange == value) return;
                _sharedXVisibleRange = value;
                OnPropertyChanged("SharedXVisibleRange");
            }
        }
        
        public bool MouseWheelEnabled
        {
            get { return _mouseWheelEnabled; }
            set
            {
                if (_mouseWheelEnabled == value) return;
                _mouseWheelEnabled = value;
                OnPropertyChanged("MouseWheelEnabled");
            }
        }

        public bool PanEnabled
        {
            get { return _panEnabled; }
            set
            {
                if (_panEnabled == value) return;
                _panEnabled = value;
                OnPropertyChanged("PanEnabled");

                // Toggle Zoom off
                ZoomEnabled = !PanEnabled;
            }
        }

        public bool ZoomEnabled
        {
            get { return _zoomEnabled; }
            set
            {
                if (_zoomEnabled == value) return;
                _zoomEnabled = value;
                OnPropertyChanged("ZoomEnabled");

                // Toggle pan off
                PanEnabled = !ZoomEnabled;
            }
        }

        public bool CursorEnabled
        {
            get { return _cursorEnabled; }
            set
            {
                if (_cursorEnabled == value) return;
                _cursorEnabled = value;
                OnPropertyChanged("CursorEnabled");

                // Toggle RolloverEnabled off
                RolloverEnabled = !CursorEnabled;
            }
        }

        public bool RolloverEnabled
        {
            get { return _rolloverEnabled; }
            set
            {
                if (_rolloverEnabled == value) return;
                _rolloverEnabled = value;
                OnPropertyChanged("RolloverEnabled");

                // Toggle RolloverEnabled off
                CursorEnabled = !RolloverEnabled;
            }
        }

        private IDataSeries CreateDataSeries1()
        {
            var ds0 = new XyDataSeries<DateTime, double>();

            const int countMin = 0;
            const int countMax = 30;
            const int count = 1000;
            for (int i = countMin; i < countMax; i++)
            {

                ds0.Append(new DateTime(1990,1,1).AddDays(i), count * Math.Sin(i * Math.PI * 0.1) / i);
            }

            return ds0;
        }

        private IDataSeries CreateDataSeries2()
        {
            var ds0 = new XyDataSeries<DateTime, double>();

            const int countMin = 20;
            const int countMax = 30;
            const int count = 1000;
            for (int i = countMin; i < countMax; i++)
            {
                ds0.Append(new DateTime(1990, 1, 1).AddDays(i), count * Math.Sin(i * Math.PI * 0.1) / i);
            }

            return ds0;
        }
    }
}
